package haifa.shopsim;

/**This is a command generated by a scheduling algorithm to a job shop simulator. It tells
the shop what to do next. Machines are identified using the machine number. 
 *@version 1.1
 */
public final class ShopCommand{

    ////////////////////////////////////////////////////////////////
    ////////////////////THE DATA//////////////////////////////
    ////////////////////////////////////////////////////////////////
    
	/**The type of command, schedule, quit, etc...*/
	protected int code=CODE_SCHEDULE;

        /**The machine that should do this command.*/
	protected int  machine;

        /**The operation to schedule.*/
        protected Operation operation;
        
    ////////////////////////////////////////////////////////////////
    ////////////////////THE CODES/////////////////////////////
    ////////////////////////////////////////////////////////////////        
        
	/**when code has this value it means that this is a scheduling command.*/
	public final static int CODE_SCHEDULE = 1;

	/**when code has this value it means that the command is to quit the work
	of the job shop.*/
	public final static int CODE_QUIT	= -1;

        /**when code has this value, it means that the command is to rest, (not do anything).*/
	public final static int CODE_REST = -10;

    ////////////////////////////////////////////////////////////////
    ////////////////////CONSTRUCTORS/////////////////////
    ////////////////////////////////////////////////////////////////        

	/**Constructs a shop command.*/
	public ShopCommand(int machine_, int code_){
		machine = machine_;
		code=code_;
	}
        
        /**Constructs a shop command with CODE_SCHEDULE*/
        public ShopCommand(int machine_,Operation operation_){
          machine=machine_;
          operation=operation_;
          //code has CODE_SCHEDULE by default
        }

        /**Set the operation.*/
        public void setOperation(Operation operation_){
            operation=operation_;
        }

        /**Returns the machine of the command.*/
        public int getMachine(){
            return machine;
        }
        
        /**Returns the operation that is to be scheduled.*/
        public Operation getOperation(){
            return operation;
        }
        
	/**Ask if the command is a quit command.*/
	public boolean isQuit(){
		return code == CODE_QUIT;
	}

        /**Ask if the command is a schedule command*/
	public boolean isSchedule(){
		return code == CODE_SCHEDULE;
	}        
        /**Ask if the command is a rest command.*/
	public boolean isRest(){
		return code == CODE_REST;
	}

        /**Returns a string with quite a bit info about the command.*/
	public String toString(){
                String codeString="";
                switch(code){
                    case CODE_SCHEDULE:
                        codeString="schedule";
                        break;
                    case CODE_REST:
                        codeString="rest";
                        break;
                    case CODE_QUIT:
                        codeString="quit";
                        break;
                } 
		return "COMMAND: "+codeString +"\tMACHINE: "+ machine + "\tOPERATION:"+operation ;
	}
}//END OF CLASS